﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using Microsoft.EntityFrameworkCore;
using BlacksmithWorkshopDatabaseImplement.Models;

namespace BlacksmithWorkshopDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new BlacksmithWorkshopDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                var deletedElement = context.Orders.Include(x => x.Manufacture).FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
                context.Orders.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
            return null;
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new BlacksmithWorkshopDatabase();
            return context.Orders.Include(x => x.Manufacture).FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && (model.DateFrom == null || model.DateTo == null))
            {
                return new();
            }
            using var context = new BlacksmithWorkshopDatabase();
            if (model.Id.HasValue)
            {
                return context.Orders.Include(x => x.Manufacture).Where(x => x.Id == model.Id).Select(x => x.GetViewModel).ToList();
            }
            else
            {
                return context.Orders.Include(x => x.Manufacture).Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                    .Select(x => x.GetViewModel).ToList();
            }
        }
        public List<OrderViewModel> GetFullList()
        {
            using var context = new BlacksmithWorkshopDatabase();
            return context.Orders.Include(x => x.Manufacture).Select(x => x.GetViewModel).ToList();
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new BlacksmithWorkshopDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders.Include(x => x.Manufacture).FirstOrDefault(x => x.Id == newOrder.Id)?.GetViewModel;
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new BlacksmithWorkshopDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return context.Orders.Include(x => x.Manufacture).FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
        }
    }
}
